#!/bin/bash

#Bugfix from DOCKER_WORKDIR behavior change
cd /root/specfem3d

# log path
mkdir -p "${SF3D_LOG}"
export LOG_FILE=${SF3D_LOG}"/sf3d.log"

source "$PATH_LOG_CONFIG"

log_info "running : `date`" 2>&1 | tee -a "${LOG_FILE}"

# sets up directory structure in current example directory
log_info "\n   setting up example...\n" 2>&1 | tee -a "${LOG_FILE}"

# cleans output folders
mkdir -p OUTPUT_FILES

mkdir -p OUTPUT_FILES/DATABASES_MPI

# clean inpput folders
mkdir -p DATA

cp -r $RESOURCES_PATH/DATA/* DATA/
cp -r $RESOURCES_PATH/MESH-default ./MESH

# stores setup
cp DATA/Par_file OUTPUT_FILES/
cp DATA/CMTSOLUTION OUTPUT_FILES/
cp DATA/STATIONS OUTPUT_FILES/

# get the number of processors, ignoring comments in the Par_file
NPROC=`grep ^NPROC DATA/Par_file | grep -v -E '^[[:space:]]*#' | cut -d = -f 2`

BASEMPIDIR=`grep ^LOCAL_PATH DATA/Par_file | cut -d = -f 2 `
mkdir -p $BASEMPIDIR

# create a hostfile for openmpi 
HOST_FILENAME="mpihost.txt"
echo "$HOSTNAME slots=$NPROC" > $HOST_FILENAME 

log_info "\n\n number of process : $NPROC\n\n" 2>&1 | tee -a "${LOG_FILE}"


# decomposes mesh using the pre-saved mesh files in MESH-default
log_info "\n\n  decomposing mesh...\n\n" 2>&1 | tee -a "${LOG_FILE}"


./bin/xdecompose_mesh $NPROC ./MESH $BASEMPIDIR
EXIT_CODE=$?
# checks exit code
if [ "$EXIT_CODE" -ne 0 ]; then
  log_info "Failed at mesh decomposing" 2>&1 | tee -a "${LOG_FILE}"
  exit $EXIT_CODE
fi

# runs database generation
if [ "$NPROC" -eq 1 ]; then
  # This is a serial simulation
  log_info "\n\n  running database generation...\n\n" 2>&1 | tee -a "${LOG_FILE}"
  ./bin/xgenerate_databases
  EXIT_CODE=$?
else
  # This is a MPI simulation
  log_info "\n\n  running database generation on $NPROC processors...\n\n" 2>&1 | tee -a "${LOG_FILE}"
  mpirun -np $NPROC --allow-run-as-root --hostfile $HOST_FILENAME ./bin/xgenerate_databases
  EXIT_CODE=$?
fi
# checks exit code
if [ "$EXIT_CODE" -ne 0 ]; then
  log_info "Failed at database generation" 2>&1 | tee -a "${LOG_FILE}"
  exit $EXIT_CODE
fi

# runs simulation
if [ "$NPROC" -eq 1 ]; then
  # This is a serial simulation
  log_info "\n\n  running solver...\n\n" 2>&1 | tee -a "${LOG_FILE}"
  ./bin/xspecfem3D
  EXIT_CODE=$?
else
  # This is a MPI simulation
  log_info "\n\n running solver on $NPROC processors...\n\n" 2>&1 | tee -a "${LOG_FILE}"
  mpirun -np $NPROC --allow-run-as-root --hostfile ./$HOST_FILENAME ./bin/xspecfem3D
  EXIT_CODE=$?
fi
# checks exit code
if [ "$EXIT_CODE" -ne 0 ]; then
  log_info "Failed at solver" 2>&1 | tee -a "${LOG_FILE}"
  exit $EXIT_CODE
fi

log_info "\n\n  see results in directory: OUTPUT_FILES/\n\n" 2>&1 | tee -a "${LOG_FILE}"

log_info "\n\n   running create movie on output data...\n\n" 2>&1 | tee -a "${LOG_FILE}"

./bin/xcreate_movie_shakemap_AVS_DX_GMT << EOF
2
1
2000
1
4
EOF
EXIT_CODE=$?

# checks exit code
if [ "$EXIT_CODE" -ne 0 ]; then
  log_info "Failed at movie file creation" 2>&1 | tee -a "${LOG_FILE}"
  exit $EXIT_CODE
fi

log_info "\n\n  moving results...\n\n" 2>&1 | tee -a "${LOG_FILE}"

# moving output files to be picked up by bucket
mv ./OUTPUT_FILES $RESOURCES_PATH

log_info "\n\n  done\n\n" 2>&1 | tee -a "${LOG_FILE}"